# Load and display NCAR Continental Outline Data, Group3, and Factmonster
# Cities.  The data are displayed in geomap item MapId in canvas MapCanvas.
# Menu items are added to MapMenu.  There widgets should exist before this
# script is sourced.

package require LibViSky

namespace eval WorldLoRes {

  # ColorScript_tcl is a script file sourced to set colors for displaying
  # map data.  It is usually created by a call to the CreateSetScript proc in
  # the LibViSky package.  Note that these are variables in the current
  # namespace.
  variable ColorDir $env(HOME)/.mapdata/colors
  variable ColorScript_tcl $ColorDir/WorldLoRes.tcl
  
  # ColorCommands will be the vars argument to CreateSetScript when called to
  # create ColorScript_tcl.  It is a list whose elements have the form
  # {colorName} or {namespaceName colorName}.  For each member of ColorCommands,
  # ColorScript_tcl will receive a line of form "set colorName $colorName"
  # or "namespace eval namespaceName "set colorName $colorName"", where
  # $colorName receives the value of the color variable when the script is
  # called.
  variable ColorCommands {}

  # Set colors from previous session.
  if [file exists $ColorScript_tcl] {
    # Set from existing ColorScript_tcl (created from previous session).
    # Make back up first in case program becomes ugly.
    Backup $ColorScript_tcl
    source $ColorScript_tcl
  }
  
  # Set map background
  lappend ColorCommands {::WorldLoRes Background}
  if ![info exists Background] {set Background #000000}
  $MapCanvas itemconfig $MapId -bg $Background

}

# Load NCAR Continental Outline Data.
namespace eval Lines::WorldLoRes {
  if ![info exists LineColor] {set LineColor #ffff99}
  lappend ::WorldLoRes::ColorCommands {::Lines::WorldLoRes LineColor}
  geolinearray fmxdr group3 GROUP3.xdr
  ${MapCanvas} itemconfig $MapId -scale [cartg 1:30000000] -center {38 -98}
  group3 draw $MapNm -c $LineColor
}

# Load cities from Tiger
namespace eval Places::WorldLoRes {
  if ![info exists PlaceColor] {set PlaceColor #ff9900}
  lappend ::WorldLoRes::ColorCommands {::Places::WorldLoRes PlaceColor}
  puts "Loading places"
  source TigerPlaces.tcl
  variable DotSize default
  # Flag to tell whether places are currently drawn.  Initialize to no.
  variable Places 0
  # Script to show places
  set ShowPlaces {
    foreach place [geoplace list] {
      $place drawdot $MapNm -c $PlaceColor -s $DotSize
    }
    ${MapMenu}.mapdata.menu entryconfigure {Show places} \
      -label "Hide places" -command [namespace code $HidePlaces]
    set ::Motion(WorldLoRes) [namespace code {
      set ::CrsInfo "$CrsInfo\nNearest place [geoplace nearest $CrsLatLon]"
    }]
    set Places 1
  }
  # Script to hide places
  set HidePlaces {
    foreach place [geoplace list] {
      $place erasedot $MapNm
    }
    if [info exists ::Motion(WorldLoRes)] {unset ::Motion(WorldLoRes)}
    ${MapMenu}.mapdata.menu entryconfigure {Hide places} \
      -label "Show places" -command [namespace code $ShowPlaces]
    set Places 0
  }
}

# Mapdata menu
menubutton ${MapMenu}.mapdata -text MapData -menu ${MapMenu}.mapdata.menu
menu ${MapMenu}.mapdata.menu
pack ${MapMenu}.mapdata -side left

namespace eval Places::WorldLoRes {
  # Additional menu items for Tiger places.

  # Menu to show places
  ${MapMenu}.mapdata.menu add command -label "Show places" \
    -command [namespace code $ShowPlaces]

  # Search for a place whose name matches a pattern.
  proc FindPlace {} [namespace code $FindPlace]
  ${MapMenu}.mapdata.menu add command -label "Find place" \
    -command [namespace code {
                set place [FindPlace]
		if {[string length $place] > 0} {
                  # Center map around place and put the '+' there.
                  ${MapCanvas} itemconfig $MapId -center $place
                  plus set [$place set]
                  if !$Places {
                    # Show places if not already doing so.
                    eval $ShowPlaces
                  }
		} else {
		  # Could not find place.  Search variable comes from the
		  # entry box.
		  tk_messageBox -message "No place named $srch"
		}
              }]

  # Dot size menu
  ${MapMenu}.mapdata.menu add cascade -label "Dot size" \
    -menu ${MapMenu}.mapdata.menu.dotsize
  menu ${MapMenu}.mapdata.menu.dotsize
  proc Resize size {
    global MapNm
    variable DotSize
    variable Places

    if {$size == "default"} {
      set DotSize $size
    } elseif {[scan $size {%d} DotSize] != 1} {
      error "Expected integer, got $size"
    }

    if $Places {
      foreach place [geoplace list] {
        $place drawdot $MapNm -s $DotSize
      }
    }

  }
  foreach size {0 1 2 3 default} {
    ${MapMenu}.mapdata.menu.dotsize add command -label $size \
      -command [namespace code "Resize $size"]
  }
}

# Menu item to adjust colors.  Cascade menu chooses which color to adjust.
${MapMenu}.mapdata.menu add cascade -label "Colors" \
  -menu ${MapMenu}.mapdata.menu.colors
menu ${MapMenu}.mapdata.menu.colors

# Menu item to adjust background color
${MapMenu}.mapdata.menu.colors add command -label "Background" \
  -command {namespace eval WorldLoRes {
               set newColor [tk_chooseColor -initialcolor $Background]
               if {[string length $newColor] > 0} {
                 set Background $newColor
	         $MapCanvas itemconfig $MapId -bg $Background
                 CreateSetScript $ColorScript_tcl $ColorCommands
               }
             }
           }

# Menu item to Adjust line color
${MapMenu}.mapdata.menu.colors add command -label "Line" \
  -command {namespace eval Lines::WorldLoRes {
               set newColor [tk_chooseColor -initialcolor $LineColor]
               if {[string length $newColor] > 0} {
                 set LineColor $newColor
                 group3 draw $MapNm -c $LineColor
                 namespace eval ::WorldLoRes {
                   CreateSetScript $ColorScript_tcl $ColorCommands
		 }
               }
             }
	   }

# Menu item to adjust place color
${MapMenu}.mapdata.menu.colors add command -label "Place" \
  -command {namespace eval Places::WorldLoRes {
               set newColor [tk_chooseColor -initialcolor $PlaceColor]
               if {[string length $newColor] > 0} {
                 set PlaceColor $newColor
                 foreach place [geoplace list] {
                   $place drawdot $MapNm -c $PlaceColor
                 }
                 namespace eval ::WorldLoRes {
                   CreateSetScript $ColorScript_tcl $ColorCommands
		 }
               }
             }
	   }
